<!DOCTYPE html>
<html>
<head>
  <style>
    header { height: 45px; position: fixed; top: 0; background-color: red; width: 100%;}
    footer { height: 45px; position: fixed; bottom: 0; background-color: red; width: 100%;}
    #main { margin: 45px;}
    #tall { display: block; height: 2000px;}
  </style>
</head>
<body>
  <header>My headers</header>
    <div id="main">
      <div id="tall">A tall block</div>
      <a href="/">Go to root</a>
    </div>
  <footer>My footer</footer>
</body>
